@@ -24,12 +24,20 @@
 
 #include "tn5250-private.h"
 
+#ifdef accept
+#undef accept
+#endif
+
 #ifdef HAVE_LIBSSL
 
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 #include <time.h>
 
+#if defined (__SVR4) && defined (__sun)
+#include <sys/filio.h>
+#endif
+
 static int ssl_stream_get_next(Tn5250Stream *This,unsigned char *buf,int size);
 static void ssl_stream_do_verb(Tn5250Stream * This, unsigned char verb, unsigned char what);
 static int ssl_stream_host_verb(Tn5250Stream * This, unsigned char verb,
@@ -176,7 +184,7 @@ int errnum;
  #define LOGERROR	ssl_logError
  #define DUMP_ERR_STACK ssl_log_error_stack
 
-static char *ssl_getTelOpt(what)
+static char *ssl_getTelOpt(unsigned char what)
 {
    char *wcp;
    static char wbuf[12];
@@ -307,7 +315,7 @@ static void ssl_log_SB_buf(unsigned char *buf, int len)
 
    if (!tn5250_logfile)
       return;
-   fprintf(tn5250_logfile,ssl_getTelOpt(type=*buf++));
+   fprintf(tn5250_logfile,"%s",ssl_getTelOpt(type=*buf++));
    switch (c=*buf++) {
       case IS:
 		fputs("<IS>",tn5250_logfile);
@@ -351,33 +359,27 @@ int tn5250_ssl_stream_init (Tn5250Stream *This)
 {
    int len;
    char methstr[5];
-   SSL_METHOD *meth=NULL;
+   const SSL_METHOD *meth=NULL;
 
    TN5250_LOG(("tn5250_ssl_stream_init() entered.\n"));
 
 /*  initialize SSL library */
 
    SSL_load_error_strings();
-   SSL_library_init();
+   OPENSSL_init_ssl(0, NULL);
 
 /*  which SSL method do we use? */
 
-   strcpy(methstr,"auto");
-   if (This->config!=NULL && tn5250_config_get (This->config, "ssl_method")) {
-        strncpy(methstr, tn5250_config_get (This->config, "ssl_method"), 4);
-        methstr[4] = '\0';
-   }
-
-   if (!strcmp(methstr, "ssl2")) {
-        meth = SSLv2_client_method();         
-        TN5250_LOG(("SSL Method = SSLv2_client_method()\n"));
-   } else if (!strcmp(methstr, "ssl3")) {
-        meth = SSLv3_client_method();         
-        TN5250_LOG(("SSL Method = SSLv3_client_method()\n"));
-   } else {
-        meth = SSLv23_client_method();         
-        TN5250_LOG(("SSL Method = SSLv23_client_method()\n"));
-   }
+    /* Ignore the user's choice of ssl_method (which isn't documented
+     * anyway...) if it was either "ssl2" or "ssl3". Both are insecure,
+     * and this is only safe supported method left.
+     *
+     * This is a Gentoo-specific modification that lets us build
+     * against LibreSSL and newer OpenSSL with its insecure protocols
+     * disabled.
+     */
+    meth = SSLv23_client_method();
+    TN5250_LOG(("SSL Method = SSLv23_client_method()\n"));
 
 /*  create a new SSL context */
 
@@ -473,99 +475,6 @@ int tn5250_ssl_stream_init (Tn5250Stream *This)
    return 0; /* Ok */
 }
 
-/****f* lib5250/tn3270_ssl_stream_init
- * NAME
- *    tn3270_ssl_stream_init
- * SYNOPSIS
- *    ret = tn3270_ssl_stream_init (This);
- * INPUTS
- *    Tn5250Stream *       This       - 
- * DESCRIPTION
- *    DOCUMENT ME!!!
- *****/
-int tn3270_ssl_stream_init (Tn5250Stream *This)
-{
-   int len;
-
-/* initialize SSL library */
-
-   SSL_load_error_strings();
-   SSL_library_init();
-
-/* create a new SSL context */
-
-   This->ssl_context = SSL_CTX_new(SSLv23_client_method());
-   if (This->ssl_context==NULL) {
-        DUMP_ERR_STACK ();
-        return -1;
-   }
-
-/* if a certificate authority file is defined, load it into this context */
-
-   if (This->config!=NULL && tn5250_config_get (This->config, "ssl_ca_file")) {
-        if (SSL_CTX_load_verify_locations(This->ssl_context, 
-                  tn5250_config_get (This->config, "ssl_ca_file"), NULL)<1) {
-            DUMP_ERR_STACK ();
-            return -1;
-        }
-   }
-
-/* if a certificate authority file is defined, load it into this context */
-
-   if (This->config!=NULL && tn5250_config_get (This->config, "ssl_ca_file")) {
-        if (SSL_CTX_load_verify_locations(This->ssl_context, 
-                  tn5250_config_get (This->config, "ssl_ca_file"), NULL)<1) {
-            DUMP_ERR_STACK ();
-            return -1;
-        }
-   }
-
-   This->userdata = NULL;
-
-/* if a PEM passphrase is defined, set things up so that it can be used */
-
-   if (This->config!=NULL && tn5250_config_get (This->config,"ssl_pem_pass")){
-        TN5250_LOG(("SSL: Setting password callback\n"));
-        len = strlen(tn5250_config_get (This->config, "ssl_pem_pass"));
-        This->userdata = malloc(len+1);
-        strncpy(This->userdata,
-                tn5250_config_get (This->config, "ssl_pem_pass"), len);
-        SSL_CTX_set_default_passwd_cb(This->ssl_context,
-                (pem_password_cb *)ssl_stream_passwd_cb);
-        SSL_CTX_set_default_passwd_cb_userdata(This->ssl_context, (void *)This);
-
-   }
-
-/* If a certificate file has been defined, load it into this context as well */
-
-   if (This->config!=NULL && tn5250_config_get (This->config, "ssl_cert_file")){
-        TN5250_LOG(("SSL: Loading certificates from certificate file\n"));
-        if (SSL_CTX_use_certificate_file(This->ssl_context,
-                tn5250_config_get (This->config, "ssl_cert_file"),
-                SSL_FILETYPE_PEM) <= 0) {
-            DUMP_ERR_STACK ();
-            return -1;
-        }
-        TN5250_LOG(("SSL: Loading private keys from certificate file\n"));
-        if (SSL_CTX_use_PrivateKey_file(This->ssl_context,
-                tn5250_config_get (This->config, "ssl_cert_file"),
-                SSL_FILETYPE_PEM) <= 0) {
-            DUMP_ERR_STACK ();
-            return -1;
-        }
-   }
-
-   This->ssl_handle = NULL;
-   This->connect = ssl_stream_connect;
-   This->accept = ssl_stream_accept;
-   This->disconnect = ssl_stream_disconnect;
-   This->handle_receive = ssl_stream_handle_receive;
-   This->send_packet = tn3270_ssl_stream_send_packet;
-   This->destroy = ssl_stream_destroy;
-   This->streamtype = TN3270E_STREAM;
-   return 0; /* Ok */
-}
-
 /****i* lib5250/ssl_stream_connect
  * NAME
  *    ssl_stream_connect
@@ -581,7 +490,7 @@ int tn3270_ssl_stream_init (Tn5250Stream *This)
 static int ssl_stream_connect(Tn5250Stream * This, const char *to)
 {
    struct sockaddr_in serv_addr;
-   u_long ioctlarg = 1;
+   int ioctlarg = 1;
    char *address;
    int r;
    X509 *server_cert;
@@ -602,7 +511,7 @@ static int ssl_stream_connect(Tn5250Stream * This, const char *to)
    if (serv_addr.sin_addr.s_addr == INADDR_NONE) {
       struct hostent *pent = gethostbyname(address);
       if (pent != NULL)
-	 serv_addr.sin_addr.s_addr = *((u_long *) (pent->h_addr));
+	 memcpy(&serv_addr.sin_addr.s_addr, pent->h_addr, 4);
    }
    free (address);
    if (serv_addr.sin_addr.s_addr == INADDR_NONE) {
@@ -739,7 +648,7 @@ static int ssl_stream_accept(Tn5250Stream * This, SOCKET_TYPE masterfd)
    int negotiating;
 
 #ifndef WINELIB
-   u_long ioctlarg=1L;
+   int ioctlarg=1;
 #endif
 
 /* FIXME:  This routine needs to be converted to use SSL calls 
@@ -1144,7 +1053,7 @@ static void ssl_stream_host_sb(Tn5250Stream * This, UCHAR *sb_buf,
 	  for(i=0; i<sb_len && sb_buf[i] != IAC; i++)
 	    tn5250_buffer_append_byte(&tbuf, sb_buf[i]);
 	  tn5250_buffer_append_byte(&tbuf, TN3270E_CONNECT);
-	  tn5250_buffer_append_data(&tbuf, dummyname, strlen(dummyname));
+	  tn5250_buffer_append_data(&tbuf, (unsigned char*) dummyname, strlen(dummyname));
 	  tn5250_buffer_append_byte(&tbuf, IAC);
 	  tn5250_buffer_append_byte(&tbuf, SE);
 	  rc = SSL_write(This->ssl_handle, (char *) tn5250_buffer_data(&tbuf),
