@@ -21,6 +21,14 @@
  */
 #include "tn5250-private.h"
 
+#if defined (__SVR4) && defined (__sun)
+#include <sys/filio.h>
+#endif
+
+#ifdef accept
+#undef accept
+#endif
+
 static int telnet_stream_get_next(Tn5250Stream * This, unsigned char *buf, int size);
 static void telnet_stream_do_verb(Tn5250Stream * This, unsigned char verb, unsigned char what);
 static int telnet_stream_host_verb(Tn5250Stream * This, unsigned char verb,
@@ -158,7 +166,7 @@ static const UCHAR SB_Str_TermType[]={IAC, SB, TERMINAL_TYPE, SEND, IAC, SE};
  #define TNSB_LOG	log_SB_buf
  #define LOGERROR	logError
 
-static char *getTelOpt(what)
+static char *getTelOpt(unsigned char what)
 {
    char *wcp;
    static char wbuf[12];
@@ -282,7 +290,7 @@ static void log_SB_buf(unsigned char *buf, int len)
 
    if (!tn5250_logfile)
       return;
-   fprintf(tn5250_logfile,getTelOpt(type=*buf++));
+   fprintf(tn5250_logfile,"%s",getTelOpt(type=*buf++));
    switch (c=*buf++) {
       case IS:
 		fputs("<IS>",tn5250_logfile);
@@ -371,7 +379,7 @@ int tn3270_telnet_stream_init (Tn5250Stream *This)
 static int telnet_stream_connect(Tn5250Stream * This, const char *to)
 {
    struct sockaddr_in serv_addr;
-   u_long ioctlarg = 1;
+   int ioctlarg = 1;
    char *address;
    int r;
 
@@ -387,8 +395,9 @@ static int telnet_stream_connect(Tn5250Stream * This, const char *to)
    serv_addr.sin_addr.s_addr = inet_addr(address);
    if (serv_addr.sin_addr.s_addr == INADDR_NONE) {
       struct hostent *pent = TN_GETHOSTBYNAME(address);
-      if (pent != NULL)
-	 serv_addr.sin_addr.s_addr = *((u_long *) (pent->h_addr));
+      if (pent != NULL) {
+         memcpy(&serv_addr.sin_addr.s_addr, pent->h_addr, 4);
+      }
    }
    free (address);
    if (serv_addr.sin_addr.s_addr == INADDR_NONE)
@@ -454,7 +463,7 @@ static int telnet_stream_accept(Tn5250Stream * This, SOCKET_TYPE masterfd)
    struct timeval tv;
 
 #ifndef WINELIB
-   u_long ioctlarg=1L;
+   int ioctlarg=1;
 #endif
 
    /*
@@ -848,7 +857,7 @@ static void telnet_stream_host_sb(Tn5250Stream * This, UCHAR *sb_buf,
 	  for(i=0; i<sb_len && sb_buf[i] != IAC; i++)
 	    tn5250_buffer_append_byte(&tbuf, sb_buf[i]);
 	  tn5250_buffer_append_byte(&tbuf, TN3270E_CONNECT);
-	  tn5250_buffer_append_data(&tbuf, dummyname, strlen(dummyname));
+	  tn5250_buffer_append_data(&tbuf, (unsigned char*)dummyname, strlen(dummyname));
 	  tn5250_buffer_append_byte(&tbuf, IAC);
 	  tn5250_buffer_append_byte(&tbuf, SE);
 	  rc = TN_SEND(This->sockfd, (char *) tn5250_buffer_data(&tbuf),
