@@ -24,6 +24,7 @@
  */
 
 #include "tn5250-private.h"
+#include "scs-private.h"
 #include <syslog.h>
 
 /* If getopt.h exists then getopt_long() probably does as well.  If
@@ -66,6 +67,7 @@
 void scs2pdf_nl (Tn5250SCS * This);
 void scs2pdf_pp (Tn5250SCS * This);
 void scs2pdf_ff (Tn5250SCS * This);
+void scs2pdf_avpp (Tn5250SCS * This);
 int scs2pdf_ahpp (Tn5250SCS * This, int *boldchars);
 void scs2pdf_cr (Tn5250SCS * This);
 void scs2pdf_ssld (Tn5250SCS * This);
@@ -376,7 +378,7 @@ main (int argc, char **argv)
   scs->data->objcount++;
   scs->data->filesize += pdf_begin_stream (scs, COURIER);
 #ifdef DEBUG
-  fprintf (stderr, "objcount = %d\n", scs->data->objcount);
+  fprintf (stderr, "objcount = %lu\n", scs->data->objcount);
 #endif
 
 
@@ -468,7 +470,7 @@ main (int argc, char **argv)
 				    scs->data->pagenumber);
   pageparent = scs->data->objcount;
 #ifdef DEBUG
-  fprintf (stderr, "pages objcount = %d\n", scs->data->objcount);
+  fprintf (stderr, "pages objcount = %lu\n", scs->data->objcount);
 #endif
 
   for (i = 0; i < scs->data->pagenumber; i++)
@@ -484,7 +486,7 @@ main (int argc, char **argv)
 				       scs->data->pdfleftmargin,
 				       scs->data->pdftopmargin);
 #ifdef DEBUG
-      fprintf (stderr, "page objcount = %d\n", scs->data->objcount);
+      fprintf (stderr, "page objcount = %lu\n", scs->data->objcount);
 #endif
     }
 
@@ -548,6 +550,7 @@ tn5250_scs2pdf_new ()
   scs->sld = scs2pdf_sld;
   scs->setfont = scs2pdf_setfont;
   /*scs->process2b = scs2pdf_process2b; */
+  scs->avpp = scs2pdf_avpp;
   scs->scsdefault = scs2pdf_default;
   return scs;
 }
@@ -648,6 +651,7 @@ scs2pdf_nl (Tn5250SCS * This)
 {
   float leftmargin;
   int leftmarginint;
+  int currentcol;
   char text[256] = {'\0'};
 
   if (This->data->newpage == 1)
@@ -655,6 +659,7 @@ scs2pdf_nl (Tn5250SCS * This)
       do_newpage (This);
     }
 
+  currentcol = This->column;
   scs_nl (This);
 
   /* If the current position on the page (column) is further to the
@@ -662,9 +667,9 @@ scs2pdf_nl (Tn5250SCS * This)
    * increase our length.  This is used later to see if we need a
    * larger page width if the page width is unspecified.
    */
-  if (This->column > This->data->columncheck)
+  if (currentcol > This->data->columncheck)
     {
-      This->data->columncheck = This->column;
+      This->data->columncheck = currentcol;
     }
 
   leftmargin = (This->leftmargin - 1) / 1440.0;
@@ -677,7 +682,6 @@ scs2pdf_nl (Tn5250SCS * This)
   sprintf (text, "%d -%d Td", leftmarginint, 72 / This->lpi);
   fprintf (outfile, "%s\n", text);
   This->data->streamsize += (strlen(text) + 1);
-  This->column = 1;
   return;
 }
 
@@ -687,7 +691,6 @@ void
 scs2pdf_ff (Tn5250SCS * This)
 {
   scs_ff (This);
-  This->column = 1;
   This->data->newpage = 1;
   return;
 }
@@ -717,7 +720,7 @@ scs2pdf_pp (Tn5250SCS * This)
       }
     case SCS_AVPP:
       {
-	scs_avpp (This);
+	This->avpp (This);
 	break;
       }
     case SCS_AHPP:
@@ -759,6 +762,67 @@ scs2pdf_pp (Tn5250SCS * This)
 }
 
 
+/* Absolute Vertical move (AVPP).  This is part of Cursor Controls.
+ */
+void
+scs2pdf_avpp (Tn5250SCS * This)
+{
+  int i;
+  int newrow;
+  int lines;
+
+  newrow = fgetc (stdin);
+
+  if ((This->usesyslog) && (This->loglevel > 0))
+    {
+      syslog (LOG_INFO,
+	      "PP sent absolute vertical move of %d (cursor currently on row %d)",
+	      newrow, This->row);
+    }
+
+#ifdef DEBUG
+  fprintf (stderr, "AVPP %d\n", newrow);
+#endif
+
+  if (newrow < This->row)
+    {
+      /* From the IPDS and SCS Technical Reference:
+       * Absolute vertical moves above the current cursor position cause
+       * the current page to be printed and the cursor to be positioned at
+       * that line on the next page.  An absolute vertical move to line 1
+       * guarantees that the printer is on a page boundary and will not
+       * cause a form feed of a blank sheet if the printer is already on
+       * line 1.  Absolute vertical moves below the bottom margin trigger
+       * a new page.
+       *
+       * So it appears that if newrow is less than This->row we should send
+       * a form feed.  However, in practice that doesn't work.  Simply
+       * sending a form feed whenever newrow is less than This->row results
+       * in extra form feeds being sent.  I am unsure of exactly what
+       * conditions are supposed to trigger a form feed as described above.
+       */
+      /*This->ff (This);*/
+      This->row = newrow;
+    }
+  else
+    {
+      lines = newrow - This->row;
+
+      if ((This->usesyslog) && (This->loglevel > 0))
+	{
+	  syslog (LOG_INFO, "Forcing %d new lines", lines);
+	}
+
+      for (i = 0; i < lines; i++)
+	{
+	  scs2pdf_nl (This);
+	}
+      This->row = newrow;
+    }
+  return;
+}
+
+
 
 /* This function is different than what is in scs.c because we want to pass
  * a pointer to the number of bold characters to print.  Since this program
@@ -1039,9 +1103,9 @@ pdf_begin_stream (Tn5250SCS * This, int fontname)
   int topmargin;
 
   sprintf (text1,
-	   "%d 0 obj\n"
+	   "%lu 0 obj\n"
 	   "\t<<\n"
-	   "\t\t/Length %d 0 R\n"
+	   "\t\t/Length %lu 0 R\n"
 	   "\t>>\n"
 	   "stream\n", This->data->objcount, This->data->objcount + 1);
   fprintf (outfile, "%s", text1);
