@@ -1,5 +1,5 @@
 /* TN5250 - An implementation of the 5250 telnet protocol.
- * Copyright (C) 2001 Scott Klement
+ * Copyright (C) 2001-2008 Scott Klement
  *
  * This file is part of TN5250
  * 
@@ -54,6 +54,7 @@ struct _Tn5250Printer {
      FILE *fileh;
      unsigned char prevchar;
      Tn5250CharMap *map;
+     char *cmd;
 };
 typedef struct _Tn5250Printer Tn5250Printer;
 
@@ -372,9 +373,10 @@ Tn5250Printer * tn5250_windows_printer_startdoc(void) {
     DOC_INFO_1 di;
     Tn5250Printer *This;
     LPTSTR devname;
-    int isfile=0;
+    int isfile=0, iscmd=0;
     int len;
     char *temp;
+    char *cmd=NULL;
 
     devname = malloc(MAXDEVNAME+1);
 
@@ -389,6 +391,11 @@ Tn5250Printer * tn5250_windows_printer_startdoc(void) {
              strncpy(devname, &temp[5], MAXDEVNAME);
              isfile=1;
         }
+        else if (!strncasecmp(temp, "exec:", 5)) {
+             strncpy(devname, &temp[5], MAXDEVNAME);
+             isfile=1;
+             iscmd=1;
+        }
         else {
              strncpy(devname, temp, MAXDEVNAME);
         }
@@ -417,6 +424,16 @@ Tn5250Printer * tn5250_windows_printer_startdoc(void) {
     This->curline = 1;
     This->prevchar = 0;
     This->count = 0;
+    This->cmd = NULL;
+
+    if (iscmd) {
+        cmd = strchr(devname, '?');
+        if (cmd!=NULL) {
+           This->cmd = malloc(strlen(cmd+1)+1);
+           strcpy(This->cmd, cmd+1);
+           *cmd = '\0';
+        }
+    }
 
     if (isfile) {
         This->fileh = fopen(devname, "w");
@@ -490,6 +507,10 @@ int tn5250_windows_printer_finishdoc(Tn5250Printer *This) {
     else if (This->fileh!=NULL) {
         fclose(This->fileh);
     }
+    if (This->cmd != NULL) {
+       win32_run_process(This->cmd, 1);
+       free(This->cmd);
+    }
     tn5250_char_map_destroy (This->map);
     free(This);
     return 0;
@@ -1168,7 +1189,7 @@ static int tn5250_windows_print_session_waitevent(Tn5250PrintSession * This)
    while ( GetMessage (&msg, NULL, 0, 0) ) {
           TranslateMessage (&msg);
           DispatchMessage(&msg);
-          if (msg.message = WM_TN5250_STREAM_DATA) {
+          if (msg.message == WM_TN5250_STREAM_DATA) {
               result = 1;
               break;
           }
